import 'dart:io';

import 'package:args/args.dart';
import 'package:get_cli/file_model.dart';
import 'package:get_cli/structure.dart';
import 'package:http/http.dart';
import 'package:recase/recase.dart';
import 'models/get_controller.dart';
import 'models/get_binding.dart';
import 'models/get_route.dart';
import 'models/stateful.dart';
import 'models/stateless.dart';

/// This is an defaultGenerator which takes [defaultStructure] as an parameter.
Generator defaultGenerator = Generator(currentStructure: defaultStructure);

/// Generator takes an [Structure] object, to specify where to create .dart files
class Generator {
  final Structure currentStructure;

  Generator({
    this.currentStructure,
  }) : assert(currentStructure != null);

  /// This method creates the file in expected path and writes inside a file as expected.
  ///
  /// if user do not write --stful flag the files which will be created will extend from StatelessWidget
  ///
  ///
  ///       get page --name LoginPage --stful
  ///
  /// the following is same
  ///
  ///       get page --name LoginPage
  ///
  ///       get page --name LoginPage  --stless
  ///
  Future handleFileCreate(ArgResults results) async {
    await defaultGenerator.createFile(
        fileModel: FileModel(
      result: results['name'],
      path: Structure.getPathWithName(
        defaultStructure.getPathByKey(results.command.name),
        ReCase(results['name']).snakeCase,
        createWithWrappedFolder: results['extraFolder'],
      ),
      commandName: results.command.name,
      isStateful: results.command['stful'],
    ));
  }

  Future<File> getOnlineModel(String path) async {
    final file = File(path)
        .writeAsBytes(await readBytes('https://picsum.photos/200/300/?random'));
    return file;
  }

  /// This function only handles file creation
  Future<File> createFile({FileModel fileModel}) async {
    var reCase = ReCase(fileModel.result);
    var _file = await File(fileModel.path + '.dart').create(recursive: true);

    await _file.writeAsString(
      fileModel.isStateful
          ? getStateFul(reCase.pascalCase)
          : getStateLess(reCase.pascalCase),
    );

    print(
        'file created succesfully with name :${fileModel.result} at path: ${fileModel.path}');
    return _file;
  }

  Future<void> createcontroller(ArgResults results) async {
    FileModel _fileModel = FileModel(
      result: results['name'],
      path: Structure.getPathWithName(
        defaultStructure.getPathByKey(results.command.name),
        ReCase(results['name']).snakeCase,
        createWithWrappedFolder: true,
      ),
      commandName: results.command.name,
      iscontroller: true,
    );
    ReCase reCase = ReCase(_fileModel.result);
    File _controller = await File(_fileModel.path + "_controller.dart")
        .create(recursive: true);
    await _controller.writeAsString(getController(reCase.pascalCase));
    File _event =
        await File(_fileModel.path + "_event.dart").create(recursive: true);
    await _event.writeAsString(getBinding(reCase.pascalCase));
    File _state =
        await File(_fileModel.path + "_state.dart").create(recursive: true);
    await _state.writeAsString(getRoutes(reCase.pascalCase));

    print(reCase.pascalCase + " controller created succesfully.");
  }
}

/// This function takes the results from the [ArgParser] and act as expected.
Future<void> generate({
  ArgResults results,
  Generator generator,
}) async {
  switch (results.command.name) {
    case "page":
      await generator.handleFileCreate(results);
      break;
    case "widget":
      await generator.handleFileCreate(results);
      break;
    case "repository":
      await generator.handleFileCreate(results);
      break;
    case "model":
      await generator.handleFileCreate(results);
      break;
    case "controller":
      await generator.createcontroller(results);
      break;
  }
}
